@@ -1,26 +1,13 @@
-/* $Log: access.c,v $
- * Revision 1.2  1993/03/19  20:37:44  bryan
- * *** empty log message ***
- *
- */
-
-static char rcsid[] = "$Id: access.c,v 1.2 1993/03/19 20:37:44 bryan Exp bryan $";
-
 #include "config.h"
 
 #include <stdio.h>
 #include <errno.h>
-#include <malloc.h>
 #include <string.h>
 #include <syslog.h>
 #include <time.h>
 #include <ctype.h>
 #include <pwd.h>
-#ifdef NeXT3
-#include <bsd/grp.h>
-#else
 #include <grp.h>
-#endif
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -37,12 +24,14 @@ static char rcsid[] = "$Id: access.c,v 1.2 1993/03/19 20:37:44 bryan Exp bryan $
 extern char remotehost[],
   remoteaddr[],
  *aclbuf;
-extern int fnmatch(),
-  nameserved,
+extern int nameserved,
   anonymous,
   guest,
   use_accessfile;
-char shutdown[MAXPATHLEN];
+char Shutdown[MAXPATHLEN];
+int pidfd = -1;
+
+extern int fnmatch();
 
 /*************************************************************************/
 /* FUNCTION  : parse_time                                                */
@@ -144,7 +133,7 @@ hostmatch(char *addr)
     if (isdigit(*addr))
         return (fnmatch(addr, remoteaddr, NULL));
     else
-        return (fnmatch(addr, remotehost, NULL));
+        return (fnmatch(addr, remotehost, FNM_NOCASE));
 }
 
 /*************************************************************************/
@@ -435,8 +424,7 @@ acl_deny(char *msgpathbuf)
 int
 acl_countusers(char *class)
 {
-    int pidfd,
-      count,
+    int count,
       which;
     char pidfile[MAXPATHLEN];
     pid_t buf[MAXUSERS];
@@ -444,19 +432,29 @@ acl_countusers(char *class)
 struct flock arg;
 #endif
 
+    /* 
+     * if pidfd was not opened previously... 
+     * pidfd must stay open after the chroot(~ftp)  
+     */
+
     sprintf(pidfile, _PATH_PIDNAMES, class);
-    pidfd = open(pidfile, O_RDWR | O_CREAT, 0644);
+
+    if (pidfd < 0) {
+        pidfd = open(pidfile, O_RDWR | O_CREAT, 0644);
+    }
+
     if (pidfd < 0) {
         syslog(LOG_ERR, "open of pid file failed: %s", strerror(errno));
         return -1;
     }
+
 #ifdef HAVE_FLOCK
     while (flock(pidfd, LOCK_EX)) {
         syslog(LOG_ERR, "sleeping: flock of pid file failed: %s",
 #else 
 	arg.l_type = F_WRLCK;
 	arg.l_whence = arg.l_start = arg.l_len = 0;
-	while ( -1 == fcntl( pidfd, F_SETLK, arg) ) {
+	while ( -1 == fcntl( pidfd, F_SETLK, &arg) ) {
 		syslog(LOG_ERR, "sleeping: fcntl lock of pid file failed: %s",
 #endif
                strerror(errno));
@@ -475,10 +473,8 @@ struct flock arg;
     flock(pidfd, LOCK_UN);
 #else
 	arg.l_type = F_UNLCK; arg.l_whence = arg.l_start = arg.l_len = 0;
-	fcntl(pidfd, F_SETLK, arg);
+	fcntl(pidfd, F_SETLK, &arg);
 #endif
-    close(pidfd);
-
     return (count);
 }
 
@@ -492,34 +488,34 @@ struct flock arg;
 void
 acl_join(char *class)
 {
-    int pidfd,
-      which,
+    int which,
       avail;
-    char pidfile[MAXPATHLEN];
     pid_t buf[MAXUSERS];
+	pid_t procid;
 #ifndef HAVE_FLOCK
 	struct flock arg;
 #endif
 
-    sprintf(pidfile, _PATH_PIDNAMES, class);
-    pidfd = open(pidfile, O_RDWR | O_CREAT, 0644);
     if (pidfd < 0) {
         syslog(LOG_ERR, "open of pid file failed: %s", strerror(errno));
         return;
     }
+
 #ifdef HAVE_FLOCK
     while (flock(pidfd, LOCK_EX)) {
         syslog(LOG_ERR, "sleeping: flock of pid file failed: %s",
 #else 
 	arg.l_type = F_WRLCK;
 	arg.l_whence = arg.l_start = arg.l_len = 0;
-	while ( -1 == fcntl( pidfd, F_SETLK, arg) ) {
+	while ( -1 == fcntl( pidfd, F_SETLK, &arg) ) {
 		syslog(LOG_ERR, "sleeping: fcntl lock of pid file failed: %s",
 #endif
                strerror(errno));
         sleep(1);
     }
 
+	procid = getpid();
+
     lseek(pidfd, 0, L_SET);
     if (read(pidfd, buf, sizeof(buf)) < sizeof(buf))
         for (which = 0; which < MAXUSERS; buf[which++] = 0)
@@ -530,10 +526,13 @@ acl_join(char *class)
         if ((buf[which] == 0) || (kill(buf[which], 0) == -1)) {
             avail = which;
             buf[which] = 0;
-        }
+        } else if (buf[which] == procid) {
+			/* already exists in pid file... */
+			return;
+		}
     }
 
-    buf[avail] = getpid();
+    buf[avail] = procid;
 
     lseek(pidfd, 0, L_SET);
     write(pidfd, buf, sizeof(buf));
@@ -541,9 +540,8 @@ acl_join(char *class)
     flock(pidfd, LOCK_UN);
 #else
 	arg.l_type = F_UNLCK; arg.l_whence = arg.l_start = arg.l_len = 0;
-	fcntl(pidfd, F_SETLK, arg);
+	fcntl(pidfd, F_SETLK, &arg);
 #endif
-    close(pidfd);
 
 }
 
@@ -590,10 +588,10 @@ access_init(void)
         return;
     (void) parseacl();
 
-    shutdown[0] = '\0';
+    Shutdown[0] = '\0';
     entry = (struct aclmember *) NULL;
     if (getaclentry("shutdown", &entry) && ARG0 != NULL)
-        (void) strncpy(shutdown, ARG0, sizeof(shutdown));
+        (void) strncpy(Shutdown, ARG0, sizeof(Shutdown));
 
 }
 
@@ -641,7 +639,7 @@ access_ok(int msgcode)
         syslog(LOG_INFO, "ACCESS DENIED (user limit %d; class %s) TO %s [%s]",
                limit, class, remotehost, remoteaddr);
         pr_mesg(msgcode, msgfile);
-        return (0);
+        return (-1);
     }
 
     /* NOTREACHED */
